import { PaymentMethodToken } from '@primer-io/checkout-web'
import Vue from 'vue'
import Component from 'vue-class-component'
import {Watch} from 'vue-property-decorator'

const onTokenizeSuccessMixinProps = Vue.extend({
  props: {
    tokenizeSuccessStatus: {
      type: [Boolean, Error],
      required: false
    },

    tokenizeSuccess: {
      type: Function,
      required: false
    }
  },

})

type PromiseResolver = {
  resolve: (value: unknown) => void;
  reject: (err: Error) => void;
}

@Component
export class onTokenizeSuccessMixin extends onTokenizeSuccessMixinProps {
  onTokenizeSuccessPromise: PromiseResolver | undefined = undefined

  clearOnTokenizeSuccess() {
    this.onTokenizeSuccessPromise = undefined
    this.$emit('tokenize-success-finish')
  }

  isWaitingForOnTokenizeSuccess() {
    return !!this.onTokenizeSuccessPromise
  }

  resolveOnTokenizeSuccess(err: Error) {
    if (!this.isWaitingForOnTokenizeSuccess()) {
      return
    }

    if (err === null || err === undefined) {
      this.onTokenizeSuccessPromise?.resolve(undefined)
    } else if (err instanceof Error) {
      this.onTokenizeSuccessPromise?.reject(err)
    }

    this.clearOnTokenizeSuccess()
  }

  onTokenizeSuccess(arg: PaymentMethodToken) {
    // Using function props
    if (this.tokenizeSuccess) {
      return this.tokenizeSuccess(arg)
    }

    // Promise to return
    const promise = new Promise((resolve, reject) => {
      this.onTokenizeSuccessPromise = {resolve, reject}
    })

    // Create callback to pass to event
    const next = (err: Error) => {
      this.resolveOnTokenizeSuccess(err)
    }

    this.$emit('tokenize-success', arg, next)

    return promise
  }

  @Watch('tokenizeSuccessStatus')
  onTokenizeSuccessStatusChange(tokenizeSuccessStatus: boolean | Error) {
    if (!this.isWaitingForOnTokenizeSuccess()) {
      return
    }

    if (tokenizeSuccessStatus === true) {
      this.onTokenizeSuccessPromise?.resolve(undefined)
      this.clearOnTokenizeSuccess()
    } else if (tokenizeSuccessStatus instanceof Error) {
      this.onTokenizeSuccessPromise?.reject(tokenizeSuccessStatus)
      this.clearOnTokenizeSuccess()
    }
  }

}


